
// Druino L/C Meter with Battery Level Indicator - Arduino Sketch          

//This Arduino sketch measures capacitance and inductance using an L/C meter and displays the results on an OLED display. It also includes a battery level indicator to monitor the battery voltage.

// Features:
// Capacitance measurement using a charging resistor and time measurement.
// Inductance measurement using a coil and pulseIn function.
// Battery level indicator based on voltage readings.

// Pin Configuration:
// analogPin: Analog pin for measuring the capacitor voltage.
// chargePin: Pin to charge the capacitor.
// dischargePin: Pin to discharge the capacitor.
// sensorPin: Analog pin for reading battery voltage.
// buttonPin: Digital input pin for the button.

// Libraries Used:
// Wire.h: I2C communication library.
// U8glib.h: Library for OLED display.
// Adafruit_SSD1306.h: Library for SSD1306 OLED display.
// Adafruit_SH1106.h: Library for SH1106 OLED display.

// How it Works:
// 1. The code initializes the required pins and the OLED display.
// 2. It toggles between capacitance and inductance measurement modes based on a button press.
// 3. In capacitance measurement mode:
   //The code charges the capacitor and measures the time it takes to reach a threshold voltage.
   //It calculates the capacitance value using the elapsed time and charging resistor value.
   //The OLED display shows the capacitance value.
   //The capacitor is then discharged.
// 4. In inductance measurement mode:
   // The code charges and discharges a coil and measures the pulse duration.
   // It calculates the inductance value using the pulse duration and a fixed capacitance value.
   // The OLED display shows the frequency and inductance value.
// 5. The battery level is measured by reading the battery voltage using the sensorPin.
   // The battery level is displayed on the OLED screen as a percentage.
// 6. The loop function repeats these measurements and updates the display accordingly.

// Note: Ensure the necessary libraries are installed before uploading the code to your Arduino board.


#include <Wire.h>
#include <U8glib.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_SH1106.h>

  U8GLIB_SSD1306_128X32 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK | U8G_I2C_OPT_FAST);
//U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK | U8G_I2C_OPT_FAST);  
//U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_DEV_0 | U8G_I2C_OPT_NO_ACK | U8G_I2C_OPT_FAST);

#define analogPin      A0               // analog pin for measuring capacitor voltage
#define chargePin      11              // pin to charge the capacitor - connected to one end of the charging resistor
#define dischargePin   8              // pin to discharge the capacitor
#define resistorValue  1000000.0F   // change this to whatever resistor value you are using
#define DEBUG_DISPLAY

unsigned long startTime = 0;
unsigned long elapsedTime = 0;

float microFarads = 0.0;                // floating point variable to preserve precision, make calculations
float nanoFarads = 0.0;
float rawV = 0.0;                    //a variable with a decimal point to hold battery voltage

int batC = 0;                       //a variable with no decimal point to hold battery percentage
int sensorPin = A1;                // Value for this example

const int buttonPin = 5;                // Digital input pin for the button

bool isInductanceMeasurement = true;  // Initial measurement mode is inductance
bool measurementTaken = false;

void draw(void) {

        // Define the battery symbol size and position
    int batteryX = 80; // X-coordinate of the battery symbol
    int batteryY = 0;  // Y-coordinate of the battery symbol
    int batteryWidth = 15; // Width of the battery symbol
    int batteryHeight = 7; // Height of the battery symbol

        // Define the maximum length for the battery level indicator
    int maxBatteryLength = batteryWidth - 2; // Adjust as needed

        // Draw battery symbol
    u8g.drawFrame(batteryX, batteryY, batteryWidth, batteryHeight);
    u8g.drawLine(batteryX + batteryWidth, batteryY + 2, batteryX + batteryWidth, batteryY + 4);

    int inputValue = analogRead(sensorPin);
    float rawV = (inputValue * 5.0) / 1024.0; // Calculate the battery voltage from the raw analog input

    int batteryLevel = 0;

    if (rawV <= 3.5) {
        batteryLevel = 0;
  } else if (rawV >= 4.0) {
       batteryLevel = 100;
  } else {
       batteryLevel = map(rawV * 10, 35, 40, 0, 100);
 }

       // Calculate the length of the battery level indicator within the specified maximum length
    int batteryIndicatorLength = map(batteryLevel, 0, 100, 0, maxBatteryLength);

      // Draw the battery level indicator box
    u8g.drawBox(batteryX + 1, batteryY + 1, batteryIndicatorLength, batteryHeight - 2);

    u8g.setFont(u8g_font_6x12);
    u8g.setPrintPos(100, 7);
    u8g.print(batteryLevel);
    u8g.drawStr(120, 7, "%");

   if (microFarads > 1.0) {
    nanoFarads = microFarads * 1000.0;
    u8g.setFont(u8g_font_6x12);
    u8g.setFontPosTop();
    u8g.setPrintPos(0, 0);   
    u8g.print(nanoFarads, 2);
    u8g.print(" nF");
  } else {
    u8g.setFont(u8g_font_6x12);
    u8g.setFontPosTop();
    u8g.setPrintPos(0, 0);
    u8g.print("10nF to 10uF");
  }

    u8g.setFont(u8g_font_fur14);        // Set the font for displaying text on the OLED display
    u8g.setPrintPos(0, 30);            // Set the starting position for printing text
    u8g.print("Cx: ");
    
       // Print the capacitance value on the OLED display  
  if (microFarads < 0.01) {
    microFarads = 0.00;  // Set to 0 for values below the threshold
    u8g.print("Unknown !");
    } else if (microFarads > 10.0) {
      u8g.print("To Large!");
  } else {
      // Print the capacitance value on the OLED display
    if (microFarads >= 1.0) {
      u8g.print(microFarads, 2);  // Print with two decimal points
      u8g.print(" uF");
    } else {
      nanoFarads = microFarads * 1000.0;
      u8g.print(nanoFarads, 2);  // Print with two decimal points
      u8g.print(" nF");
      delay(50);
    }
  }
}

void setup() {
  #ifdef DEBUG_DISPLAY
  u8g.firstPage();
  do {
    u8g.setFont(u8g_font_9x15);
    u8g.setFontPosTop();
    u8g.setPrintPos(20, 0);
    u8g.print("HM Projects");
    u8g.setPrintPos(35, 16);
    u8g.print("L/C Meter");
  } while (u8g.nextPage());
  #endif

  delay(3000);

  if (u8g.getMode() == U8G_MODE_R3G3B2)
    u8g.setColorIndex(255);
  else if (u8g.getMode() == U8G_MODE_GRAY2BIT)
    u8g.setColorIndex(3);
  else if (u8g.getMode() == U8G_MODE_BW)
    u8g.setColorIndex(1);
  else if (u8g.getMode() == U8G_MODE_HICOLOR)
    u8g.setHiColorByRGB(255, 255, 255);

  pinMode(3, INPUT);
  pinMode(12, OUTPUT);
  pinMode(chargePin, OUTPUT);     // set chargePin to output
  digitalWrite(chargePin, LOW);
  pinMode(buttonPin, INPUT_PULLUP);
}

void loop() {  
  
  if (digitalRead(buttonPin) == LOW) {
    // Button is pressed, toggle the measurement mode
    isInductanceMeasurement = !isInductanceMeasurement;
    delay(200);  // Add a small delay to debounce the button
  }

  if (isInductanceMeasurement) {
    double pulse, frequency, capacitance, inductance;
    digitalWrite(12, HIGH);
    delay(5);
    digitalWrite(12, LOW);
    delayMicroseconds(100);
    pulse = pulseIn(3, HIGH, 5000);

    if (pulse > 0.1) {
      // Replace the capacitance value with the one you are using
      capacitance = 1.E-6;

      frequency = 1.E6 / (2 * pulse);
      inductance = 1. / (capacitance * frequency * frequency * 4. * 3.14159 * 3.14159);
      inductance *= 1E6;
      
      measurementTaken = true;
    } else {
      capacitance = 0.0;   // No capacitance measurement
      inductance = 0.0;   // No inductance measurement
      measurementTaken = false;
    }

    char inductanceUnit[3];

    if (inductance >= 1000) {
      inductance /= 1000;
      strcpy(inductanceUnit, "mH");
    } else {
      strcpy(inductanceUnit, "uH");
    }

    u8g.firstPage();
    do {
      
        // Define the battery symbol size and position
    int batteryX = 80; // X-coordinate of the battery symbol
    int batteryY = 0;  // Y-coordinate of the battery symbol
    int batteryWidth = 15; // Width of the battery symbol
    int batteryHeight = 7; // Height of the battery symbol

        // Define the maximum length for the battery level indicator
    int maxBatteryLength = batteryWidth - 2; // Adjust as needed

        // Draw battery symbol
    u8g.drawFrame(batteryX, batteryY, batteryWidth, batteryHeight);
    u8g.drawLine(batteryX + batteryWidth, batteryY + 2, batteryX + batteryWidth, batteryY + 4);

    int inputValue = analogRead(sensorPin);
    float rawV = (inputValue * 5.0) / 1024.0; // Calculate the battery voltage from the raw analog input

    int batteryLevel = 0;

    if (rawV <= 3.5) {
        batteryLevel = 0;
  } else if (rawV >= 4.0) {
       batteryLevel = 100;
  } else {
       batteryLevel = map(rawV * 10, 35, 40, 0, 100);
 }

       // Calculate the length of the battery level indicator within the specified maximum length
    int batteryIndicatorLength = map(batteryLevel, 0, 100, 0, maxBatteryLength);

      // Draw the battery level indicator box
    u8g.drawBox(batteryX + 1, batteryY + 1, batteryIndicatorLength, batteryHeight - 2);

    u8g.setFont(u8g_font_6x12);
    u8g.setPrintPos(100, 7);
    u8g.print(batteryLevel);
    u8g.drawStr(120, 7, "%");

    if (measurementTaken) {
        u8g.setFont(u8g_font_6x12);
        u8g.setFontPosTop();
        u8g.setPrintPos(0, 0);
        u8g.print(frequency);
        u8g.print(" Hz");
     }else {
        u8g.setFont(u8g_font_6x12);
        u8g.setFontPosTop();
        u8g.setPrintPos(0, 0);
        u8g.print("Waiting..");
     }
      u8g.setFont(u8g_font_fur14);
      u8g.setFontPosTop();
      u8g.setPrintPos(0, 15);
      if (measurementTaken) {
        u8g.print("Lx:");
        u8g.setPrintPos(30, 15);
        u8g.print(inductance, 3);
        u8g.print(" ");
        u8g.print(inductanceUnit);
      } else {
        u8g.print("Lx: Unknown !");
      }
      delay(50);
    } while (u8g.nextPage());
  } else {
    digitalWrite(chargePin, HIGH);  // set chargePin HIGH and capacitor charging
  startTime = millis();

  while (analogRead(analogPin) < 648) {
    // Wait until the voltage reaches the threshold
  }

  elapsedTime = millis() - startTime;

  // convert milliseconds to seconds (10^-3) and Farads to microFarads (10^6), net 10^3 (1000)
  microFarads = ((float)elapsedTime / resistorValue) * 1000;

  // Clear the display before updating
  u8g.firstPage();
  do {
    draw();                       // Call the draw() function to update the OLED display
  } while (u8g.nextPage());

  /* dicharge the capacitor  */
  digitalWrite(chargePin, LOW);               // set charge pin to LOW
  pinMode(dischargePin, OUTPUT);             // set discharge pin to output
  digitalWrite(dischargePin, LOW);          // set discharge pin LOW

  while (analogRead(analogPin) > 0) {
    // wait until capacitor is completely discharged
  }

  pinMode(dischargePin, INPUT);            // set discharge pin back to input
 }
}
